/**
 * Activate the plugin license.
 * @returns {Promise<string>} A promise that resolves to the path of the captured document image.
 */
export declare function activateLicense(androidLic: string, iosLic: string): Promise<string>;
/**
 * Initializes the Quickcapture module.
 * @returns {boolean} Returns true to indicate successful initialization.
 */
export declare function initialise(): boolean;
/**
 * Captures a document using the device camera.
 * @returns {Promise<string>} A promise that resolves to the path of the captured document image.
 */
export declare function captureDocument(): Promise<string>;
/**
 * Builds a PDF from the last captured document.
 * @returns {Promise<string>} A promise that resolves to the file path of the generated PDF.
 */
export declare function buildPdfForLastDocumentCapture(): Promise<string>;
/**
 * Captures an optical code (QR code or barcode) using the device camera.
 * @returns {Promise<string>} A promise that resolves to the scanned code data.
 */
export declare function captureOpticalCode(): Promise<string>;
/**
 * Generates a QR code from provided data.
 * @param {string} data The data to encode in the QR code.
 * @returns {Promise<string>} A promise that resolves to Base64 data of the generated QR code image.
 */
export declare function generateQRCode(data: string): Promise<string>;
/**
 * Generates a barcode from provided data.
 * @param {string} data The data to encode in the barcode.
 * @param {boolean} includeText Optional flag to include text below the barcode.
 * @returns {Promise<string>} A promise that resolves to Base64 data of the generated QR code image.
 */
export declare function generateBarcode(data: string, includeText?: boolean): Promise<string>;
//# sourceMappingURL=index.d.ts.map